@@ -2,8 +2,9 @@
 A low level example:
 This is how JenkinsAPI creates views
 """
-import requests
+from __future__ import print_function
 import json
+import requests
 
 url = 'http://localhost:8080/createView'
 
@@ -14,8 +15,10 @@ data = {
     "name": str_view_name,
     "mode": "hudson.model.ListView",
     "Submit": "OK",
-    "json": json.dumps({"name": str_view_name, "mode": "hudson.model.ListView"})
+    "json": json.dumps(
+        {"name": str_view_name, "mode": "hudson.model.ListView"}
+    )
 }
 # Try 1
 result = requests.post(url, params=params, data=data, headers=headers)
-print result.text.encode('UTF-8')
+print(result.text.encode('UTF-8'))
