@@ -1,6 +1,10 @@
 """
 Module for jenkinsapi Job
 """
+import json
+import logging
+import xml.etree.ElementTree as ET
+import six.moves.urllib.parse as urlparse
 
 from collections import defaultdict
 from jenkinsapi.build import Build
@@ -16,17 +20,7 @@ from jenkinsapi.custom_exceptions import (
 from jenkinsapi.jenkinsbase import JenkinsBase
 from jenkinsapi.mutable_jenkins_thing import MutableJenkinsThing
 from jenkinsapi.queue import QueueItem
-import json
-import logging
-
-import xml.etree.ElementTree as ET
-
-
-try:
-    import urlparse
-except ImportError:
-    # Python3
-    import urllib.parse as urlparse
+from jenkinsapi_utils.compat import to_string
 
 
 SVN_URL = './scm/locations/hudson.scm.SubversionSCM_-ModuleLocation/remote'
@@ -70,7 +64,8 @@ class Job(JenkinsBase, MutableJenkinsThing):
             'hg': self._get_hg_branch,
             None: lambda element_tree: []
         }
-        JenkinsBase.__init__(self, url)
+        self.url = url
+        JenkinsBase.__init__(self, self.url)
 
     def __str__(self):
         return self.name
@@ -95,10 +90,10 @@ class Job(JenkinsBase, MutableJenkinsThing):
 
     def poll(self, tree=None):
         data = super(Job, self).poll(tree=tree)
-        if not tree:
+        if not tree and not self.jenkins.lazy:
             self._data = self._add_missing_builds(self._data)
-        else:
-            return data
+
+        return data
 
     # pylint: disable=E1123
     # Unexpected keyword arg 'params'
@@ -115,7 +110,7 @@ class Job(JenkinsBase, MutableJenkinsThing):
         # do not call _buildid_for_type here: it would poll and do an infinite
         # loop
         oldest_loaded_build_number = data["builds"][-1]["number"]
-        if not self._data['firstBuild']:
+        if 'firstBuild' not in self._data or not self._data['firstBuild']:
             first_build_number = oldest_loaded_build_number
         else:
             first_build_number = self._data["firstBuild"]["number"]
@@ -138,11 +133,8 @@ class Job(JenkinsBase, MutableJenkinsThing):
             self._element_tree = ET.fromstring(self._config)
         return self._element_tree
 
-    def get_build_triggerurl(self, files, build_params=None):
-        if (files and build_params) or (not self.has_params()):
-            # If job has file parameters and non-file parameters - it must be
-            # triggered using "/build", not by "/buildWithParameters"
-            # "/buildWithParameters" will ignore non-file parameters
+    def get_build_triggerurl(self):
+        if not self.has_params():
             return "%s/build" % self.baseurl
         return "%s/buildWithParameters" % self.baseurl
 
@@ -153,11 +145,12 @@ class Job(JenkinsBase, MutableJenkinsThing):
         Key-Value pairs would be far too simple, no no!
         Watch and read on and behold!
         """
-        assert isinstance(
-            build_params, dict), 'Build parameters must be a dict'
+        if not isinstance(build_params, dict):
+            raise ValueError('Build parameters must be a dict')
 
-        build_p = [{'name': k, 'value': str(v)}
+        build_p = [{'name': k, 'value': to_string(v)}
                    for k, v in sorted(build_params.items())]
+
         out = {'parameter': build_p}
         if file_params:
             file_p = [{'name': k, 'file': k}
@@ -191,10 +184,10 @@ class Job(JenkinsBase, MutableJenkinsThing):
             params['token'] = securitytoken
 
         # Either copy the params dict or make a new one.
-        build_params = build_params and dict(
-            build_params.items()) or {}  # Via POSTed JSON
+        build_params = dict(build_params.items()) \
+            if build_params else {}  # Via POSTed JSON
 
-        url = self.get_build_triggerurl(files, build_params)
+        url = self.get_build_triggerurl()
         if cause:
             build_params['cause'] = cause
 
@@ -218,13 +211,27 @@ class Job(JenkinsBase, MutableJenkinsThing):
 
         redirect_url = response.headers['location']
 
-        if not redirect_url.startswith("%s/queue/item" % self.jenkins.baseurl):
-            if files:
-                raise ValueError('Builds with file parameters are not '
-                                 'supported by this jenkinsapi version. '
-                                 'Please use previous version.')
-            else:
-                raise ValueError("Not a Queue URL: %s" % redirect_url)
+        #
+        # Enterprise Jenkins implementations such as CloudBees locate their
+        # queue REST API base https://server.domain.com/jenkins/queue/api/
+        # above the team-specific REST API base
+        # https://server.domain.com/jenkins/job/my_team/api/
+        #
+        queue_baseurl_candidates = [self.jenkins.baseurl]
+        scheme, netloc, path, _, query, frag = \
+            urlparse.urlparse(self.jenkins.baseurl)
+        while path:
+            path = '/'.join(path.rstrip('/').split('/')[:-1])
+            queue_baseurl_candidates.append(
+                urlparse.urlunsplit([scheme, netloc, path, query, frag]))
+        redirect_url_valid = False
+        for queue_baseurl_candidate in queue_baseurl_candidates:
+            redirect_url_valid = redirect_url.startswith(
+                "%s/queue/item" % queue_baseurl_candidate)
+            if redirect_url_valid:
+                break
+        if not redirect_url_valid:
+            raise ValueError("Not a Queue URL: %s" % redirect_url)
 
         qi = QueueItem(redirect_url, self.jenkins)
         if block:
@@ -232,7 +239,9 @@ class Job(JenkinsBase, MutableJenkinsThing):
         return qi
 
     def _buildid_for_type(self, buildtype):
-        """Gets a buildid for a given type of build"""
+        """
+        Gets a buildid for a given type of build
+        """
         KNOWNBUILDTYPES = [
             "lastStableBuild",
             "lastSuccessfulBuild",
@@ -269,7 +278,7 @@ class Job(JenkinsBase, MutableJenkinsThing):
 
     def get_last_failed_buildnumber(self):
         """
-        Get the numerical ID of the last good build.
+        Get the numerical ID of the last failed build.
         """
         return self._buildid_for_type(buildtype="lastFailedBuild")
 
@@ -300,6 +309,22 @@ class Job(JenkinsBase, MutableJenkinsThing):
         # understand the test above.
         return dict((build["number"], build["url"]) for build in builds)
 
+    def get_build_by_params(self, build_params, order=1):
+        first_build_number = self.get_first_buildnumber()
+        last_build_number = self.get_last_buildnumber()
+        if order != 1 and order != -1:
+            raise ValueError(
+                'Direction should be ascending or descending (1/-1)')
+
+        for number in range(first_build_number,
+                            last_build_number + 1)[::order]:
+            build = self.get_build(number)
+            if build.get_params() == build_params:
+                return build
+
+        raise NoBuildData(
+            'No build with such params {params}'.format(params=build_params))
+
     def get_revision_dict(self):
         """
         Get dictionary of all revisions with a list of buildnumbers (int)
@@ -385,8 +410,26 @@ class Job(JenkinsBase, MutableJenkinsThing):
 
     def get_build(self, buildnumber):
         assert isinstance(buildnumber, int)
-        url = self.get_build_dict()[buildnumber]
-        return Build(url, buildnumber, job=self)
+        try:
+            url = self.get_build_dict()[buildnumber]
+            return Build(url, buildnumber, job=self)
+        except KeyError:
+            raise NotFound('Build #%s not found' % buildnumber)
+
+    def delete_build(self, build_number):
+        """
+        Remove build
+
+        :param int build_number:    Build number
+        :raises NotFound:           When build is not found
+        """
+        try:
+            url = self.get_build_dict()[build_number]
+            url = "%s/doDelete" % url
+            self.jenkins.requester.post_and_confirm_status(url, data='')
+            self.jenkins.poll()
+        except KeyError:
+            raise NotFound('Build #%s not found' % build_number)
 
     def get_build_metadata(self, buildnumber):
         """
@@ -394,9 +437,17 @@ class Job(JenkinsBase, MutableJenkinsThing):
         tons of tests, this method is faster than get_build by returning less
         data.
         """
-        assert isinstance(buildnumber, int)
-        url = self.get_build_dict()[buildnumber]
-        return Build(url, buildnumber, job=self, depth=0)
+        if not isinstance(buildnumber, int):
+            raise ValueError('Parameter "buildNumber" must be int')
+
+        try:
+            url = self.get_build_dict()[buildnumber]
+            return Build(url, buildnumber, job=self, depth=0)
+        except KeyError:
+            raise NotFound('Build #%s not found' % buildnumber)
+
+    def __delitem__(self, build_number):
+        self.delete_build(build_number)
 
     def __getitem__(self, buildnumber):
         return self.get_build(buildnumber)
@@ -418,7 +469,11 @@ class Job(JenkinsBase, MutableJenkinsThing):
         """
         if not self.is_queued():
             raise UnknownQueueItem()
-        return QueueItem(self.jenkins, **self._data['queueItem'])
+        q_item = self.poll(tree='queueItem[url]')
+        qi_url = urlparse.urljoin(
+            self.jenkins.baseurl, q_item['queueItem']['url']
+        )
+        return QueueItem(qi_url, self.jenkins)
 
     def is_running(self):
         # self.poll()
@@ -433,7 +488,9 @@ class Job(JenkinsBase, MutableJenkinsThing):
         return False
 
     def get_config(self):
-        '''Returns the config.xml from the job'''
+        """
+        Returns the config.xml from the job
+        """
         response = self.jenkins.requester.get_and_confirm_status(
             "%(baseurl)s/config.xml" % self.__dict__)
         return response.text
@@ -535,14 +592,7 @@ class Job(JenkinsBase, MutableJenkinsThing):
             Useful for debugging and validation workflows.
         """
         url = self.get_config_xml_url()
-        try:
-            if isinstance(
-                    config, unicode):  # pylint: disable=undefined-variable
-                config = str(config)
-        except NameError:
-            # Python3 already a str
-            pass
-
+        config = str(config)  # cast unicode in case of Python 2
         response = self.jenkins.requester.post_url(url, params={}, data=config)
         self._element_tree = ET.fromstring(config)
 
@@ -606,15 +656,19 @@ class Job(JenkinsBase, MutableJenkinsThing):
 
     def is_enabled(self):
         data = self.poll(tree='color')
-        return data.get('color', None) != 'disabled'
+        return 'disabled' not in data.get('color', '')
 
     def disable(self):
-        '''Disable job'''
+        """
+        Disable job
+        """
         url = "%s/disable" % self.baseurl
         return self.get_jenkins_obj().requester.post_url(url, data='')
 
     def enable(self):
-        '''Enable job'''
+        """
+        Enable job
+        """
         url = "%s/enable" % self.baseurl
         return self.get_jenkins_obj().requester.post_url(url, data='')
 
@@ -627,7 +681,7 @@ class Job(JenkinsBase, MutableJenkinsThing):
             raise NotInQueue()
         queue_id = self._data['queueItem']['id']
         url = urlparse.urljoin(self.get_jenkins_obj().get_queue().baseurl,
-                               'cancelItem?id=%s' % queue_id)
+                               'queue/cancelItem?id=%s' % queue_id)
         self.get_jenkins_obj().requester.post_and_confirm_status(url, data='')
         return True
 
@@ -667,14 +721,43 @@ class Job(JenkinsBase, MutableJenkinsThing):
         """
         If job has parameters, returns True, else False
         """
-        return any("parameterDefinitions" in a for a in (
-            self._data["actions"] or self._data["property"]) if a)
+        if any("parameterDefinitions" in a for a in (self._data["actions"])
+               if a):
+            return True
+        if any("parameterDefinitions" in a for a in (self._data["property"])
+               if a):
+            return True
+        return False
 
     def has_queued_build(self, build_params):
-        """Returns True if a build with build_params is currently queued."""
+        """
+        Returns True if a build with build_params is currently queued.
+        """
         queue = self.jenkins.get_queue()
         queued_builds = queue.get_queue_items_for_job(self.name)
         for build in queued_builds:
             if build.get_parameters() == build_params:
                 return True
         return False
+
+    @staticmethod
+    def get_full_name_from_url_and_baseurl(url, baseurl):
+        """
+        Get the full name for a job (including parent folders) from the
+        job URL.
+        """
+        path = url.replace(baseurl, '')
+        split = path.split('/')
+        split = [urlparse.unquote(part) for part in split[::2] if part]
+        return '/'.join(split)
+
+    def get_full_name(self):
+        """
+        Get the full name for a job (including parent folders)
+        from the job URL.
+        """
+        return Job.get_full_name_from_url_and_baseurl(
+            self.url, self.jenkins.baseurl)
+
+    def toggle_keep_build(self, build_number):
+        self.get_build(build_number).toggle_keep()
