@@ -1,12 +1,7 @@
 """
 Module for jenkinsapi views
 """
-try:
-    from urllib import urlencode
-except ImportError:
-    # Python3
-    from urllib.parse import urlencode
-
+import six
 import logging
 
 from jenkinsapi.jenkinsbase import JenkinsBase
@@ -29,9 +24,15 @@ class View(JenkinsBase):
         JenkinsBase.__init__(self, url)
         self.deleted = False
 
+    def __len__(self):
+        return len(self.get_job_dict().keys())
+
     def __str__(self):
         return self.name
 
+    def __repr__(self):
+        return self.name
+
     def __getitem__(self, job_name):
         assert isinstance(job_name, str)
         api_url = self.python_api_url(self.get_job_url(job_name))
@@ -56,15 +57,10 @@ class View(JenkinsBase):
         return self.get_job_dict().keys()
 
     def iteritems(self):
-        try:
-            it = self.get_job_dict().iteritems()
-        except AttributeError:
-            # Python3
-            it = self.get_job_dict().items()
+        it = six.iteritems(self.get_job_dict())
 
         for name, url in it:
-            api_url = self.python_api_url(url)
-            yield name, Job(api_url, name, self.jenkins_obj)
+            yield name, Job(url, name, self.jenkins_obj)
 
     def values(self):
         return [a[1] for a in self.iteritems()]
@@ -80,9 +76,6 @@ class View(JenkinsBase):
     def get_job_dict(self):
         return dict(self._get_jobs())
 
-    def __len__(self):
-        return len(self.get_job_dict().keys())
-
     def get_job_url(self, str_job_name):
         if str_job_name in self:
             return self.get_job_dict()[str_job_name]
@@ -95,19 +88,20 @@ class View(JenkinsBase):
     def get_jenkins_obj(self):
         return self.jenkins_obj
 
-    def add_job(self, str_job_name, job=None):
+    def add_job(self, job_name, job=None):
         """
         Add job to a view
 
-        :param str_job_name: name of the job to be added
+        :param job_name: name of the job to be added
         :param job: Job object to be added
         :return: True if job has been added, False if job already exists or
          job not known to Jenkins
         """
         if not job:
-            if str_job_name in self.get_job_dict():
-                log.warn(msg='Job %s is already in the view %s' %
-                         (str_job_name, self.name))
+            if job_name in self.get_job_dict():
+                log.warning(
+                    'Job %s is already in the view %s',
+                    job_name, self.name)
                 return False
             else:
                 # Since this call can be made from nested view,
@@ -116,53 +110,53 @@ class View(JenkinsBase):
                 # http://jenkins:8080/view/CRT/view/CRT-FB/view/CRT-SCRT-1301/
                 top_jenkins = self.get_jenkins_obj().get_jenkins_obj_from_url(
                     self.baseurl.split('view/')[0])
-                if not top_jenkins.has_job(str_job_name):
+                if not top_jenkins.has_job(job_name):
                     log.error(
                         msg='Job "%s" is not known to Jenkins' %
-                        str_job_name)
+                        job_name)
                     return False
                 else:
-                    job = top_jenkins.get_job(str_job_name)
+                    job = top_jenkins.get_job(job_name)
 
-        log.info(msg='Creating job %s in view %s' % (str_job_name, self.name))
-        data = {
-            "description": "",
-            "statusFilter": "",
-            "useincluderegex": "on",
-            "includeRegex": "",
-            "columns": [{"stapler-class": "hudson.views.StatusColumn",
-                         "kind": "hudson.views.StatusColumn"},
-                        {"stapler-class": "hudson.views.WeatherColumn",
-                         "kind": "hudson.views.WeatherColumn"},
-                        {"stapler-class": "hudson.views.JobColumn",
-                         "kind": "hudson.views.JobColumn"},
-                        {"stapler-class": "hudson.views.LastSuccessColumn",
-                         "kind": "hudson.views.LastSuccessColumn"},
-                        {"stapler-class": "hudson.views.LastFailureColumn",
-                         "kind": "hudson.views.LastFailureColumn"},
-                        {"stapler-class": "hudson.views.LastDurationColumn",
-                         "kind": "hudson.views.LastDurationColumn"},
-                        {"stapler-class": "hudson.views.BuildButtonColumn",
-                         "kind": "hudson.views.BuildButtonColumn"}],
-            "Submit": "OK",
-        }
-        data["name"] = self.name
-        # Add existing jobs (if any)
-        for job_name in self.get_job_dict().keys():
-            data[job_name] = 'on'
+        log.info(msg='Creating job %s in view %s' % (job_name, self.name))
 
-        # Add new job
-        data[job.name] = 'on'
+        url = '%s/addJobToView' % self.baseurl
+        params = {'name': job_name}
 
-        data['json'] = data.copy()
-        data = urlencode(data)
         self.get_jenkins_obj().requester.post_and_confirm_status(
-            '%s/configSubmit' % self.baseurl, data=data)
+            url,
+            data={},
+            params=params)
         self.poll()
         log.debug(msg='Job "%s" has been added to a view "%s"' %
                   (job.name, self.name))
         return True
 
+    def remove_job(self, job_name):
+        """
+        Remove job from a view
+
+        :param job_name: name of the job to be removed
+        :return: True if job has been removed,
+            False if job not assigned to this view
+        """
+        if job_name not in self:
+            return False
+
+        url = '%s/removeJobFromView' % self.baseurl
+        params = {'name': job_name}
+
+        self.get_jenkins_obj().requester.post_and_confirm_status(
+            url,
+            data={},
+            params=params)
+        self.poll()
+        log.debug(
+            msg='Job "%s" has been added to a view "%s"'
+            % (job_name, self.name)
+        )
+        return True
+
     def _get_nested_views(self):
         for viewdict in self._data.get("views", []):
             yield viewdict["name"], viewdict["url"]
@@ -186,13 +180,7 @@ class View(JenkinsBase):
         Update the config.xml to the view
         """
         url = self.get_config_xml_url()
-        try:
-            if isinstance(
-                    config, unicode):  # pylint: disable=undefined-variable
-                config = str(config)
-        except NameError:
-            # Python3 already a str
-            pass
+        config = str(config)  # cast unicode in case of Python 2
 
         response = self.get_jenkins_obj().requester.post_url(
             url, params={}, data=config)
